/*
Copyright 2020 The KubeEdge Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"time"

	v1alpha1 "github.com/kubeedge/kubeedge/cloud/pkg/apis/devices/v1alpha1"
	scheme "github.com/kubeedge/kubeedge/cloud/pkg/client/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// DevicesGetter has a method to return a DeviceInterface.
// A group's client should implement this interface.
type DevicesGetter interface {
	Devices(namespace string) DeviceInterface
}

// DeviceInterface has methods to work with Device resources.
type DeviceInterface interface {
	Create(*v1alpha1.Device) (*v1alpha1.Device, error)
	Update(*v1alpha1.Device) (*v1alpha1.Device, error)
	UpdateStatus(*v1alpha1.Device) (*v1alpha1.Device, error)
	Delete(name string, options *v1.DeleteOptions) error
	DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error
	Get(name string, options v1.GetOptions) (*v1alpha1.Device, error)
	List(opts v1.ListOptions) (*v1alpha1.DeviceList, error)
	Watch(opts v1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha1.Device, err error)
	DeviceExpansion
}

// devices implements DeviceInterface
type devices struct {
	client rest.Interface
	ns     string
}

// newDevices returns a Devices
func newDevices(c *DevicesV1alpha1Client, namespace string) *devices {
	return &devices{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the device, and returns the corresponding device object, and an error if there is any.
func (c *devices) Get(name string, options v1.GetOptions) (result *v1alpha1.Device, err error) {
	result = &v1alpha1.Device{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("devices").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Devices that match those selectors.
func (c *devices) List(opts v1.ListOptions) (result *v1alpha1.DeviceList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.DeviceList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("devices").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested devices.
func (c *devices) Watch(opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("devices").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a device and creates it.  Returns the server's representation of the device, and an error, if there is any.
func (c *devices) Create(device *v1alpha1.Device) (result *v1alpha1.Device, err error) {
	result = &v1alpha1.Device{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("devices").
		Body(device).
		Do().
		Into(result)
	return
}

// Update takes the representation of a device and updates it. Returns the server's representation of the device, and an error, if there is any.
func (c *devices) Update(device *v1alpha1.Device) (result *v1alpha1.Device, err error) {
	result = &v1alpha1.Device{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("devices").
		Name(device.Name).
		Body(device).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *devices) UpdateStatus(device *v1alpha1.Device) (result *v1alpha1.Device, err error) {
	result = &v1alpha1.Device{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("devices").
		Name(device.Name).
		SubResource("status").
		Body(device).
		Do().
		Into(result)
	return
}

// Delete takes name of the device and deletes it. Returns an error if one occurs.
func (c *devices) Delete(name string, options *v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("devices").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *devices) DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("devices").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched device.
func (c *devices) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha1.Device, err error) {
	result = &v1alpha1.Device{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("devices").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
